@@ -1,21 +1,30 @@
 package Tapper::MCP::Config;
+BEGIN {
+  $Tapper::MCP::Config::AUTHORITY = 'cpan:AMD';
+}
+{
+  $Tapper::MCP::Config::VERSION = '4.0.1';
+}
 
 use strict;
 use warnings;
 
 use 5.010;
+use Data::DPath 'dpath';
 use File::Basename;
 use Fcntl;
 use File::Path;
 use LockFile::Simple;
 use Moose;
-use Socket;
+use Socket 'inet_ntoa';
 use Sys::Hostname;
-use YAML;
+use YAML::Syck qw /Load Dump LoadFile DumpFile/;
 
 use Tapper::Model 'model';
 use Tapper::Config;
 use Tapper::MCP::Info;
+use Tapper::Producer;
+use Try::Tiny;
 
 extends 'Tapper::MCP::Control';
 
@@ -29,75 +38,37 @@ sub BUILD
 }
 
 
-our $MODIFIER = 3; # timeout = $MODIFIER * runtime; XXX find better way
 
-=head1 NAME
-
-Tapper::MCP::Config - Generate config for a certain test run
-
-=head1 SYNOPSIS
-
- use Tapper::MCP::Config;
-
-=head1 FUNCTIONS
-
-=cut
-
-=head2 parse_simnow_preconditions
-
-Parse a simnow precondition.
-
-@param hash ref - config
-@param hash ref - simnow precondition
-
-@return success - 0
-
-=cut
 
 sub parse_simnow_preconditions
 {
         my ($self, $config, $precondition) = @_;
-        $self->mcp_info->is_simnow(1);
+        $self->mcp_info->test_type('simnow');
         return $config;
 }
 
-=head2 parse_simnow_preconditions
-
-Parse a simnow precondition.
-
-@param hash ref - config
-@param hash ref - hint precondition
-
-@return success - 0
-
-=cut
 
 sub parse_hint_preconditions
 {
         my ($self, $config, $precondition) = @_;
         if ($precondition->{simnow}) {
-                $self->mcp_info->is_simnow(1);
+                $self->mcp_info->test_type('simnow');
                 $config->{paths}{base_dir}='/';
                 $config->{files}{simnow_script} = $precondition->{script} if $precondition->{script};
+                push @{$config->{preconditions}}, {precondition_type => 'simnow_backend'};
+        } elsif ($precondition->{ssh}) {
+                $self->mcp_info->test_type('ssh');
+                $config->{paths}{base_dir}='/';
+                $config->{prcs}->[0]->{skip_startscript} = 1;
+                $config->{client_package} = {
+                                             arch      => $precondition->{arch},
+                                             dest_path => $precondition->{dest_path},
+                                            } if $precondition->{arch};
         }
-        push @{$config->{preconditions}}, {precondition_type => 'simnow_backend'};
         return $config;
 }
 
 
-=head2 add_tapper_package_for_guest
-
-Add opt tapper package to guest
-
-@param hash ref - config
-@param hash ref - guest
-@param int - guest number
-
-
-@return success - new config (hash ref)
-@return error   - error string
-
-=cut
 
 sub add_tapper_package_for_guest
 {
@@ -105,7 +76,7 @@ sub add_tapper_package_for_guest
         my ($self, $config, $guest, $guest_number) = @_;
         my $tapper_package->{precondition_type} = '';
 
-        my $guest_arch                        = $guest->{root}{arch};
+        my $guest_arch                       = $guest->{root}{arch} or return "No architecture set for guest #$guest_number";
         $tapper_package->{filename}          = $self->cfg->{files}->{tapper_package}{$guest_arch};
 
         $tapper_package->{precondition_type} = 'package';
@@ -117,29 +88,10 @@ sub add_tapper_package_for_guest
 }
 
 
-=head2 handle_guest_tests
-
-Create guest PRC config based on guest tests.
-
-@param hash ref - old config
-@param hash ref - guest description
-@param int      - guest number
-
-@return success - new config hash ref
-@return error   - error string
-
-=cut
 
 sub handle_guest_tests
 {
         my ($self, $config, $guest, $guest_number) = @_;
-        $config = $self->add_tapper_package_for_guest($config, $guest);
-        return $config unless ref $config eq 'HASH';
-
-        $config->{prcs}->[$guest_number]->{mountfile} = $guest->{mountfile};
-        $config->{prcs}->[$guest_number]->{mountpartition} = $guest->{mountpartition};
-        $config->{prcs}->[$guest_number]->{config}->{guest_number} = $guest_number;
-
 
         $config = $self->parse_testprogram($config, $guest->{testprogram}, $guest_number)
           if $guest->{testprogram};
@@ -153,21 +105,11 @@ sub handle_guest_tests
 }
 
 
-=head2 parse_virt_host
-
-Parse host definition of a virt precondition and change config accordingly
-
-@param hash ref - old config
-@param hash ref - virt precondition
-
-@return hash ref - new config
-
-=cut
 
 sub parse_virt_host
 {
         my ($self, $config, $virt) = @_;
-        given ($virt->{host}->{root}->{precondition_type}) {
+        given (lc($virt->{host}->{root}->{precondition_type})) {
                 when ('image') {
                         $config = $self->parse_image_precondition($config, $virt->{host}->{root});
                 }
@@ -185,18 +127,6 @@ sub parse_virt_host
 
 
 
-=head2 parse_virt_preconditions
-
-Unpack a precondition virt entry into images, packages and files to be
-installed for this virt package to work.
-
-@param hash ref - config hash to which virt precondition should be added
-@param hash ref - precondition as hash
-
-@return success - hash reference containing the new config
-@return error   - error string
-
-=cut
 
 sub parse_virt_preconditions
 {
@@ -208,14 +138,17 @@ sub parse_virt_preconditions
         $config = $self->parse_testprogram($config, $virt->{host}->{testprogram}, 0) if $virt->{host}->{testprogram};
         $config = $self->parse_testprogram_list($config, $virt->{host}->{testprogram_list}, 0) if $virt->{host}->{testprogram_list};
         return $config unless ref($config) eq 'HASH';
+        my $total_guests = int @{$virt->{guests} || []};
 
         for (my $guest_number = 1; $guest_number <= int @{$virt->{guests} || []}; $guest_number++ ) {
                 my $guest = $virt->{guests}->[$guest_number-1];
 
                 $guest->{mountfile} = $guest->{root}->{mountfile};
                 $guest->{mountpartition} = $guest->{root}->{mountpartition};
+                $guest->{mountdir} = $guest->{root}->{mountdir};
                 delete $guest->{root}->{mountpartition};
                 delete $guest->{root}->{mountfile} if $guest->{root}->{mountfile};
+                delete $guest->{root}->{mountdir};
 
 
                 $retval = $self->mcp_info->add_prc($guest_number, $self->cfg->{times}{boot_timeout});
@@ -246,7 +179,7 @@ sub parse_virt_preconditions
                 use warnings;
 
                 push @{$config->{preconditions}}, $guest->{root} if $guest->{root}->{precondition_type};
-                push @{$config->{preconditions}}, $guest->{config};
+                push @{$config->{preconditions}}, $guest->{config} if exists $guest->{config}->{precondition_type};
                 if ($guest->{config}->{svm}) {
                         push @{$config->{prcs}->[0]->{config}->{guests}}, {svm=>$guest->{config}->{svm}};
                 } elsif ($guest->{config}->{kvm}) {
@@ -262,11 +195,26 @@ sub parse_virt_preconditions
 
                 # put guest preconditions into precondition list
                 foreach my $guest_precondition(@{$guest->{preconditions}}) {
-                        $guest_precondition->{mountpartition} = $guest->{mountpartition};
-                        $guest_precondition->{mountfile} = $guest->{mountfile} if $guest->{mountfile};
-                        push @{$config->{preconditions}}, $guest_precondition;
+                        if ( $guest_precondition->{precondition_type} eq 'testprogram' ) {
+                                $config = $self->parse_testprogram($config, $guest_precondition, $guest_number);
+                        } elsif ( $guest_precondition->{precondition_type} eq 'testprogram' ) {
+                                $config = $self->parse_testprogram_list($config, $guest_precondition, $guest_number);
+                        } else {
+                                $guest_precondition->{mountpartition} = $guest->{mountpartition};
+                                $guest_precondition->{mountfile} = $guest->{mountfile} if $guest->{mountfile};
+                                push @{$config->{preconditions}}, $guest_precondition;
+                        }
+                        return $config unless ref $config eq 'HASH';
                 }
 
+                # add a PRC for every guest
+                $config = $self->add_tapper_package_for_guest($config, $guest, $guest_number);
+                return $config unless ref $config eq 'HASH';
+
+                $config->{prcs}->[$guest_number]->{mountfile} = $guest->{mountfile};
+                $config->{prcs}->[$guest_number]->{mountpartition} = $guest->{mountpartition};
+                $config->{prcs}->[$guest_number]->{config}->{guest_number} = $guest_number;
+                $config->{prcs}->[$guest_number]->{config}->{total_guests} = $total_guests;
         }
         $config->{prcs}->[0]->{config}->{guest_count} = int @{$virt->{guests} || []};
 
@@ -274,44 +222,16 @@ sub parse_virt_preconditions
 }
 
 
-=head2 parse_grub
-
-Handle precondition grub. Even though a preconfigured grub config is provided
-as precondition, it needs to get a special place in the Yaml file. Otherwise
-it would be hard to find for the installer process generating the grub config
-file.
-
-@param hash ref - config to change
-@param hash ref - precondition as hash
-
-@return success - config hash
-@return error   - error string
-
-=cut
 
 sub  parse_grub
 {
-        my ($self, $config, $grub) = @_;
-        $config->{grub}=$grub->{config};
+        my ($self, $config, $precondition) = @_;
+        $config->{grub} = $precondition->{config};
         return $config;
 }
 
 
 
-=head2 parse_reboot
-
-Handle precondition grub. Even though a preconfigured grub config is provided
-as precondition, it needs to get a special place in the Yaml file. Otherwise
-it would be hard to find for the installer process generating the grub config
-file.
-
-@param hash ref - config to change
-@param hash ref - precondition as hash
-
-@return success - config hash
-@return error   - error string
-
-=cut
 
 sub  parse_reboot
 {
@@ -321,18 +241,6 @@ sub  parse_reboot
         return $config;
 }
 
-=head2 parse_image_precondition
-
-Handle precondition image. Make sure the appropriate opt-tapper package is
-installed if needed. Care for the root image being installed first.
-
-@param hash ref - config to change
-@param hash ref - precondition as hash
-
-@return success - config hash
-@return error   - error string
-
-=cut
 
 sub parse_image_precondition
 {
@@ -357,26 +265,12 @@ sub parse_image_precondition
         if ($opt_pkg) {
                 push @{$config->{preconditions}}, $opt_pkg;
                 push @{$config->{preconditions}}, {precondition_type => 'exec',
-                                                   filename => '/opt/tapper/perl/perls/current/bin/tapper-testsuite-hwtrack',
+                                                   filename          => '/opt/tapper/perl/perls/current/bin/tapper-testsuite-hwtrack',
                                                    continue_on_error => 1 };
         }
         return $config;
 }
 
-=head2 parse_testprogram
-
-Handle precondition testprogram. Make sure testprogram is correctly to config
-and internal information set.
-
-@param hash ref - config to change
-@param hash ref - precondition as hash
-@param int - prc_number, optional
-
-
-@return success - config hash
-@return error   - error string
-
-=cut
 
 sub parse_testprogram
 {
@@ -393,7 +287,7 @@ sub parse_testprogram
         }
         $testprogram->{runtime} = $testprogram->{runtime} || $self->cfg->{times}{test_runtime_default};
 
-        return "No timeout for testprogram" if not $testprogram->{timeout};
+        $testprogram->{timeout} = ($self->cfg->{times}{default_testprogram_timeout} // 600) unless defined $testprogram->{timeout};
         no warnings 'uninitialized';
         push @{$config->{prcs}->[$prc_number]->{config}->{testprogram_list}}, $testprogram;
         $self->mcp_info->add_testprogram($prc_number, $testprogram);
@@ -402,20 +296,6 @@ sub parse_testprogram
 
 }
 
-=head2 parse_testprogram_list
-
-Handle testprogram list precondition. Puts testprograms to config and
-internal information set.
-
-@param hash ref - config to change
-@param hash ref - precondition as hash
-@param int - prc_number, optional
-
-
-@return success - config hash
-@return error   - error string
-
-=cut
 
 sub parse_testprogram_list
 {
@@ -430,17 +310,6 @@ sub parse_testprogram_list
 
 
 
-=head2 parse_autoinstall
-
-Parse precondition autoinstall and change config accordingly.
-
-@param hash ref - config to change
-@param hash ref - precondition as hash
-
-@return success - config hash
-@return error   - error string
-
-=cut
 
 sub parse_autoinstall
 {
@@ -460,33 +329,198 @@ sub parse_autoinstall
         $config->{paths}{base_dir} = '/';
         my $timeout = $autoinstall->{timeout} || $self->cfg->{times}{installer_timeout};
         $self->mcp_info->set_installer_timeout($timeout);
-
-        my $tapper_host=$config->{mcp_host};
-        my $tapper_port=$config->{mcp_port};
-        my $packed_ip = gethostbyname($tapper_host);
-        if (not defined $packed_ip) {
-                return "Can not get an IP address for tapper_host ($tapper_host): $!";
-        }
-        my $tapper_ip=inet_ntoa($packed_ip);
-        my $tapper_environment = Tapper::Config::_getenv();
-        my $testrun = $config->{test_run};
-        $config->{installer_grub} =~
-          s|\$TAPPER_OPTIONS|tapper_ip=$tapper_ip tapper_host=$tapper_host tapper_environment=$tapper_environment testrun=$testrun|g;
-
         return $config;
 }
 
 
-=head2 get_install_config
+sub update_installer_grub
+{
+        my ($self, $config)    = @_;
 
-Add installation configuration part to a given config hash.
+        $config->{installer_grub} = $self->cfg->{mcp}{installer}{default_grub} if not $config->{installer_grub};
+        return $config;
+}
 
-@param hash reference - config to change
 
-@return success - config hash
-@return error   - error string
+sub produce
+{
+        my ($self, $config, $precondition) = @_;
+        my $producer = Tapper::Producer->new();
+        my $producer_config = $producer->produce($self->testrun->testrun_scheduling, $precondition);
+
+        die $producer_config if not ref($producer_config) eq 'HASH';
+
+        if ($producer_config->{topic}) {
+                $self->testrun->topic_name($producer_config->{topic});
+                $self->testrun->update;
+        }
+        my @precond_array = Load($producer_config->{precondition_yaml});
+        return \@precond_array;
+}
+
+
+
+sub parse_produce_precondition
+{
+        my ($self, $config, $precondition) = @_;
+
+        my $error;
+        my $produced_preconditions = try {$self->produce($config, $precondition->precondition_as_hash)} catch {$error = $_};
+        return $error if $error;
+
+        return $produced_preconditions
+          unless ref($produced_preconditions) eq 'ARRAY';
+        my $position = model->resultset('TestrunPrecondition')->search({testrun_id => $self->testrun->id,
+                                                                        precondition_id => $precondition->id})->first->succession;
+        $self->testrun->disassign_preconditions($precondition->id);
+
+        foreach my $produced_precondition (@$produced_preconditions) {
+                my ($new_id) = model->resultset('Precondition')->add( [$produced_precondition] );
+                $self->testrun->insert_preconditions($position++, $new_id);
+
+                my ($new_precondition) = model->resultset('Precondition')->find( $new_id );
+                $config = $self->parse_precondition($config, $new_precondition);
+                return $config unless ref($config) eq 'HASH';
+        }
+        return $config;
+
+}
+
+
+sub produce_preconds_in_arrayref
+{
+        my ($self, $config, $preconditions) = @_;
+        my @new_preconds;
+
+        my $error;
+        return "Did not receive an array ref for 'produce_preconds_in_arrayref'"
+          unless ref $preconditions eq 'ARRAY';
+
+        foreach my $precondition ( @$preconditions ) {
+                if (lc($precondition->{precondition_type}) eq 'produce') {
+                        my $produced_preconditions = try {$self->produce($config, $precondition)} catch {$error = $_};
+                        return $error if $error;
+                        push @new_preconds, @$produced_preconditions;
+                } else {
+                        push @new_preconds, $precondition;
+                }
+        }
+        @$preconditions = @new_preconds;
+        return 0;
+}
+
+
+sub produce_virt_precondition
+{
+        my ($self, $config, $precondition) = @_;
+        local $Data::DPath::USE_SAFE; # path not from user, Safe.pm deactivated for debug and speed
+        my $producers = $precondition ~~ dpath '//*[key eq "precondition_type" and lc(value) eq "produce"]/../..';
+        foreach my $producer (@$producers) {
+                if (ref $producer eq 'ARRAY') {
+                        my $error = $self->produce_preconds_in_arrayref($config, $producer);
+                        return $error if $error;
+                } elsif (ref $producer eq 'HASH') {
+                        foreach my $key ( keys %$producer ) {
+                                if (ref($producer->{$key}) eq 'ARRAY') {
+                                        my $error = $self->produce_preconds_in_arrayref($config, $producer->{$key});
+                                        return $error if $error;
+                                } elsif (ref($producer->{$key}) eq 'HASH' and
+                                         lc($producer->{$key}->{precondition_type}) eq 'produce') {
+                                        my $error;
+                                        my $produced_preconditions = try {$self->produce($config, $producer->{$key})} catch {$error = $_};
+                                        return $error if $error;
+                                        $producer->{$key} = $produced_preconditions->[0];
+                                }
+                        }
+                }
+        }
+        return $precondition;
+}
+
+
+
+sub parse_precondition
+{
+        my ($self, $config, $precondition_result) = @_;
+        my $precondition = $precondition_result->precondition_as_hash;
+
+        given(lc($precondition->{precondition_type})){
+                when('produce') {
+                        $config = $self->parse_produce_precondition($config, $precondition_result);
+                }
+                when('image' ) {
+                        $config = $self->parse_image_precondition($config, $precondition);
+                }
+                when( 'virt' ) {
+                        $precondition = $self->produce_virt_precondition($config, $precondition);
+                        return $precondition unless ref $precondition eq 'HASH';
+
+
+                        $precondition_result->precondition(Dump($precondition));
+                        $precondition_result->update;
+
+                        $config       = $self->parse_virt_preconditions($config, $precondition);
+                }
+                when( 'grub') {
+                        $config = $self->parse_grub($config, $precondition);
+                }
+                when( 'installer_stop') {
+                        $config->{installer_stop} = 1;
+                }
+                when( 'reboot') {
+                        $config = $self->parse_reboot($config, $precondition);
+                }
+                when( 'autoinstall') {
+                        $config = $self->parse_autoinstall($config, $precondition);
+                }
+                when( 'testprogram') {
+                        $config = $self->parse_testprogram($config, $precondition);
+                }
+                when( 'testprogram_list') {
+                        $config = $self->parse_testprogram_list($config, $precondition);
+                }
+                when( 'simnow' ) {
+                        $config=$self->parse_simnow_preconditions($config, $precondition);
+                }
+                when( 'hint' ) {
+                        $config=$self->parse_hint_preconditions($config, $precondition);
+                }
+                default {
+                        push @{$config->{preconditions}}, $precondition;
+                }
+        }
+
+
+        return $config;
+}
+
+# replace $TAPPER_PLACEHOLDERS in grub config file
+sub grub_substitute_variables
+{
+        no warnings 'uninitialized'; # some options may not be set, especially during testing. This is ok.
+        my ($self, $config, $grubtext) = @_;
+
+        my $tapper_host        = $config->{mcp_host};
+        my $tapper_port        = $config->{mcp_port};
+        my $packed_ip          = gethostbyname($tapper_host);
+        die "Can not get an IP address for tapper_host ($tapper_host): $!" if not defined $packed_ip;
+        my $tapper_ip          = inet_ntoa($packed_ip);
+        my $tapper_environment = Tapper::Config::_getenv();
+        my $testrun            = $config->{test_run};
+        my $nfsroot            = $config->{paths}{nfsroot};
+        my $kernel             = $config->{files}{installer_kernel};
+        my $tftp_server        = $self->cfg->{tftp_server_address};
+        my $hostoptions        = $self->cfg->{grub_completion_HOSTOPTIONS}{$config->{hostname}} || $self->cfg->{grub_completion_HOSTOPTIONS}{_default};
+
+        $grubtext =~ s|\$TAPPER_OPTIONS\b|tapper_ip=$tapper_ip tapper_port=$tapper_port testrun=$testrun tapper_host=$tapper_host tapper_environment=$tapper_environment|g;
+        $grubtext =~ s|\$TAPPER_NFSROOT\b|$nfsroot|g;
+        $grubtext =~ s|\$TAPPER_TFTPSERVER\b|$tftp_server|g;
+        $grubtext =~ s|\$TAPPER_KERNEL\b|$kernel|g;
+        $grubtext =~ s|\$HOSTOPTIONS\b|$hostoptions|g;
+
+        return $grubtext;
+}
 
-=cut
 
 sub get_install_config
 {
@@ -496,69 +530,51 @@ sub get_install_config
         my $retval = $self->mcp_info->add_prc(0, $self->cfg->{times}{boot_timeout});
         return $retval if $retval;
 
-        foreach my $precondition ($self->testrun->ordered_preconditions) {
-                # make sure installing the root partition is always the first precondition
-                if ($precondition->precondition_as_hash->{precondition_type} eq 'image' ) {
-                        $config = $self->parse_image_precondition($config, $precondition->precondition_as_hash);
-                }
-                elsif ($precondition->precondition_as_hash->{precondition_type} eq 'virt' ) {
-                        $config=$self->parse_virt_preconditions($config, $precondition->precondition_as_hash);
-                }
-                elsif ($precondition->precondition_as_hash->{precondition_type} eq 'grub') {
-                        $config = $self->parse_grub($config, $precondition->precondition_as_hash);
-                }
-                elsif ($precondition->precondition_as_hash->{precondition_type} eq 'installer_stop') {
-                        $config->{installer_stop} = 1;
-                }
-                elsif ($precondition->precondition_as_hash->{precondition_type} eq 'reboot') {
-                        $config = $self->parse_reboot($config, $precondition->precondition_as_hash);
-                }
-                elsif ($precondition->precondition_as_hash->{precondition_type} eq 'autoinstall') {
-                        $config = $self->parse_autoinstall($config, $precondition->precondition_as_hash);
-                }
-                elsif ($precondition->precondition_as_hash->{precondition_type} eq 'testprogram') {
-                        $config = $self->parse_testprogram($config, $precondition->precondition_as_hash);
-                }
-                elsif ($precondition->precondition_as_hash->{precondition_type} eq 'testprogram_list') {
-                        $config = $self->parse_testprogram_list($config, $precondition->precondition_as_hash);
-                }
-                elsif ($precondition->precondition_as_hash->{precondition_type} eq 'simnow' ) {
-                        $config=$self->parse_simnow_preconditions($config, $precondition->precondition_as_hash);
-                }
-                elsif ($precondition->precondition_as_hash->{precondition_type} eq 'hint' ) {
-                        $config=$self->parse_hint_preconditions($config, $precondition->precondition_as_hash);
-                }
-                else {
-                        push @{$config->{preconditions}}, $precondition->precondition_as_hash;
-                }
+        {
+                no warnings 'uninitialized'; # allowing this timeout to be undef is a feature
+                $retval    = $self->mcp_info->set_keep_alive_timeout($self->cfg->{keep_alive}{timeout_receive});
+                $config->{times}{keep_alive_timeout} = $self->cfg->{keep_alive}{timeout_send};
+                $config->{mcp_callback_handler}{plugin} = $self->cfg->{mcp_callback_handler}{plugin};
+                $config->{mcp_callback_handler}{plugin_plugin} = $self->cfg->{mcp_callback_handler}{plugin_options};
+        }
 
+ PRECONDITION:
+        foreach my $precondition_result ( $self->testrun->ordered_preconditions) {
+                $config = $self->parse_precondition($config, $precondition_result);
                 # was not able to parse precondition and thus
                 # return received error string
                 if (not ref($config) eq 'HASH' ) {
                         return $config;
                 }
         }
+
+
+
         # always have a PRC0 even without any test programs
-        unless ($self->mcp_info->is_simnow() or $config->{prcs}) {
+        unless ($self->mcp_info->test_type() eq 'simnow'
+                or $config->{prcs}) {
                 $config->{prcs}->[0] = {testprogram_list => []};
         }
 
+        # generate installer config
+        $config = $self->update_installer_grub($config);
+
         while (my $prc_precondition = shift(@{$config->{prcs}})){
                 $prc_precondition->{precondition_type} = "prc";
                 push(@{$config->{preconditions}}, $prc_precondition);
         }
+
+        $config->{grub} = $self->cfg->{mcp}{test}{default_grub} if not $config->{grub};
+
+        my $error;
+        $config->{installer_grub} = try { $self->grub_substitute_variables($config, $config->{installer_grub}) }
+          catch { $error = $_} if $config->{installer_grub}; return $error if $error;
+        $config->{grub}           = try { $self->grub_substitute_variables($config, $config->{grub}) }
+          catch { $error = $_} if ($config->{grub}); return $error if $error;
         return $config;
 }
 
 
-=head2 get_common_config
-
-Create configuration to be used for installation on a given host.
-
-@return success - config hash reference
-@return error   - error string
-
-=cut
 
 sub get_common_config
 {
@@ -581,6 +597,10 @@ sub get_common_config
         $config->{test_run}                  = $testrun->id;
         $config->{testrun_id}                = $testrun->id;
 
+        if ($testrun->testplan_id) {
+                $config->{testplan} = { id => $testrun->testplan_id, path => $testrun->testplan_instance->path };
+        }
+
         if ($self->testrun->scenario_element) {
                 $config->{scenario_id} = $self->testrun->scenario_element->scenario_id;
                 my $path = $config->{paths}{sync_path}."/".$config->{scenario_id}."/";
@@ -604,42 +624,19 @@ sub get_common_config
                                 print $fh $self->testrun->scenario_element->peer_elements->count;
                                 close $fh;
                         }       # else trust the creator
-                        eval {
+                        my $error;
+                        try {
                                 YAML::DumpFile($config->{files}{sync_file}, \@peers);
-                        };
-                        return $@ if $@;
+                        } catch { $error = $_};
+                        return $error if $error;
+
                 }
         }
         return ($config);
 }
 
 
-=head2 get_mcp_info
 
-Returns mcp_info attribute, no matter if its already set.
-
-@return hash reference
-
-=cut
-
-sub get_mcp_info
-{
-        my ($self) = @_;
-
-        return $self->mcp_info;
-}
-
-
-=head2 get_test_config
-
-Returns a an array of configs for all PRCs of a given test. All information
-are taken from the MCP::Info attribute of the object so its only save to call
-this function after create_config which configures this attribute.
-
-@return success - config array (array ref)
-@return error   - error string
-
-=cut
 
 sub get_test_config
 {
@@ -654,15 +651,6 @@ sub get_test_config
 }
 
 
-=head2 create_config
-
-Create a configuration for the current status of the test machine. All config
-information are taken from the database based upon the given testrun id.
-
-@return success - config (hash reference)
-@return error   - error string
-
-=cut
 
 sub create_config
 {
@@ -674,17 +662,6 @@ sub create_config
         return $config;
 }
 
-=head2 write_config
-
-Write the config created before into appropriate YAML file.
-
-@param string - config (hash reference)
-@param string - output file name, in absolut form or relative to configured localdata_path
-
-@return success - 0
-@return error   - error string
-
-=cut
 
 sub write_config
 {
@@ -700,6 +677,268 @@ sub write_config
 
 1;
 
+
+__END__
+=pod
+
+=encoding utf-8
+
+=head1 NAME
+
+Tapper::MCP::Config
+
+=head1 SYNOPSIS
+
+ use Tapper::MCP::Config;
+
+=head1 NAME
+
+Tapper::MCP::Config - Generate config for a certain test run
+
+=head1 FUNCTIONS
+
+=head2 parse_simnow_preconditions
+
+Parse a simnow precondition.
+
+@param hash ref - config
+@param hash ref - simnow precondition
+
+@return success - 0
+
+=head2 parse_hint_preconditions
+
+Parse a hint precondition.
+
+@param hash ref - config
+@param hash ref - hint precondition
+
+@return success - 0
+
+=head2 add_tapper_package_for_guest
+
+Add opt tapper package to guest
+
+@param hash ref - config
+@param hash ref - guest
+@param int - guest number
+
+@return success - new config (hash ref)
+@return error   - error string
+
+=head2 handle_guest_tests
+
+Create guest PRC config based on guest tests.
+
+@param hash ref - old config
+@param hash ref - guest description
+@param int      - guest number
+
+@return success - new config hash ref
+@return error   - error string
+
+=head2 parse_virt_host
+
+Parse host definition of a virt precondition and change config accordingly
+
+@param hash ref - old config
+@param hash ref - virt precondition
+
+@return hash ref - new config
+
+=head2 parse_virt_preconditions
+
+Unpack a precondition virt entry into images, packages and files to be
+installed for this virt package to work.
+
+@param hash ref - config hash to which virt precondition should be added
+@param hash ref - precondition as hash
+
+@return success - hash reference containing the new config
+@return error   - error string
+
+=head2 parse_grub
+
+Handle precondition grub. Even though a preconfigured grub config is provided
+as precondition, it needs to get a special place in the Yaml file. Otherwise
+it would be hard to find for the installer process generating the grub config
+file.
+
+@param hash ref - config to change
+@param hash ref - precondition as hash
+
+@return success - config hash
+@return error   - error string
+
+=head2 parse_reboot
+
+Handle precondition grub. Even though a preconfigured grub config is provided
+as precondition, it needs to get a special place in the Yaml file. Otherwise
+it would be hard to find for the installer process generating the grub config
+file.
+
+@param hash ref - config to change
+@param hash ref - precondition as hash
+
+@return success - config hash
+@return error   - error string
+
+=head2 parse_image_precondition
+
+Handle precondition image. Make sure the appropriate opt-tapper package is
+installed if needed. Care for the root image being installed first.
+
+@param hash ref - config to change
+@param hash ref - precondition as hash
+
+@return success - config hash
+@return error   - error string
+
+=head2 parse_testprogram
+
+Handle precondition testprogram. Make sure testprogram is correctly to config
+and internal information set.
+
+@param hash ref - config to change
+@param hash ref - precondition as hash
+@param int - prc_number, optional
+
+@return success - config hash
+@return error   - error string
+
+=head2 parse_testprogram_list
+
+Handle testprogram list precondition. Puts testprograms to config and
+internal information set.
+
+@param hash ref - config to change
+@param hash ref - precondition as hash
+@param int - prc_number, optional
+
+@return success - config hash
+@return error   - error string
+
+=head2 parse_autoinstall
+
+Parse precondition autoinstall and change config accordingly.
+
+@param hash ref - config to change
+@param hash ref - precondition as hash
+
+@return success - config hash
+@return error   - error string
+
+=head2 update_installer_grub
+
+Get the text for grub config file at booting into installation.
+
+@param hash ref - config to change
+
+@return success - config hash
+@return error   - error string
+
+=head2 produce
+
+Calls the producer for the given precondition
+
+@param hash ref - config
+@param hash ref - precondition
+
+@return success - array ref containing preconditions
+
+@throws die()
+
+=head2 parse_produce_precondition
+
+Parse a producer precondition, insert the produced ones and delete the
+old one. In case of success the updated config is returned.
+
+@param hash ref                   - old config
+@param precondition result object - precondition
+
+@return success - hash ref
+@return error   - error string
+
+=head2 produce_preconds_in_arrayref
+
+Take an array ref, find the producers in it and produce them. Substitute
+the producer preconditions with the produced preconditions they generated.
+
+This function changes the received argument instead of returning an
+updated version. This makes sure you can change your precondition step
+by step instead of having to create a new one.
+
+@param hash ref  - config
+@param array ref - preconditions with producers
+
+@return success - 0
+@return error   - error string
+
+=head2 produce_virt_precondition
+
+Find all producers in a virt precondition, call them and substitute the
+producer preconditions with the received produced preconditions. It
+returns the updated virt precondition.
+
+@param hash ref - config
+@param hash ref - precondition as hash
+
+@return success - hash ref containing updated precondition
+@return error   - error string
+
+=head2 parse_precondition
+
+Parse a given precondition and update the config accordingly.
+
+@param hash ref                   - old config
+@param precondition result object - precondition
+
+@return success - hash ref containing updated config
+@return error   - error string
+
+=head2 get_install_config
+
+Add installation configuration part to a given config hash.
+
+@param hash reference - config to change
+
+@return success - config hash
+@return error   - error string
+
+=head2 get_common_config
+
+Create configuration to be used for installation on a given host.
+
+@return success - config hash reference
+@return error   - error string
+
+=head2 get_test_config
+
+Returns a an array of configs for all PRCs of a given test. All information
+are taken from the MCP::Info attribute of the object so its only save to call
+this function after create_config which configures this attribute.
+
+@return success - config array (array ref)
+@return error   - error string
+
+=head2 create_config
+
+Create a configuration for the current status of the test machine. All config
+information are taken from the database based upon the given testrun id.
+
+@return success - config (hash reference)
+@return error   - error string
+
+=head2 write_config
+
+Write the config created before into appropriate YAML file.
+
+@param string - config (hash reference)
+@param string - output file name, in absolut form or relative to configured localdata_path
+
+@return success - 0
+@return error   - error string
+
 =head1 AUTHOR
 
 AMD OSRC Tapper Team, C<< <tapper at amd64.org> >>
@@ -714,12 +953,25 @@ You can find documentation for this module with the perldoc command.
 
  perldoc Tapper
 
-
 =head1 ACKNOWLEDGEMENTS
 
-
 =head1 COPYRIGHT & LICENSE
 
 Copyright 2008-2011 AMD OSRC Tapper Team, all rights reserved.
 
 This program is released under the following license: freebsd
+
+=head1 AUTHOR
+
+AMD OSRC Tapper Team <tapper@amd64.org>
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is Copyright (c) 2012 by Advanced Micro Devices, Inc..
+
+This is free software, licensed under:
+
+  The (two-clause) FreeBSD License
+
+=cut
+
