@@ -1,14 +1,21 @@
 package Tapper::MCP::Net;
+BEGIN {
+  $Tapper::MCP::Net::AUTHORITY = 'cpan:AMD';
+}
+{
+  $Tapper::MCP::Net::VERSION = '4.0.1';
+}
 
 use strict;
 use warnings;
+use English '-no_match_vars';
 
 use 5.010;
 
 use Moose;
 use Socket;
 use Net::SSH;
-use Net::SSH::Expect;
+use Net::SCP;
 use IO::Socket::INET;
 use Sys::Hostname;
 use File::Basename;
@@ -18,20 +25,6 @@ extends 'Tapper::MCP';
 
 use Tapper::Model qw(model get_hardware_overview);
 
-=head2 conserver_connect
-
-This function opens a connection to the conserver. Conserver, port and user
-can be given as arguments, yet are optional.
-@param string - system to open a console to
-@opt   string - Address or name of the console server
-@opt   int    - port number of the console server
-@opt   string - username to be used
-
-
-@returnlist success - (IO::Socket::INET object)
-@returnlist error   - (error string)
-
-=cut
 
 sub conserver_connect
 {
@@ -53,9 +46,13 @@ sub conserver_connect
         print $sock "call $system\n";
         my $port=<$sock>;
         if ($port=~ /@(\w+)/) {
+                close $sock;
                 return $self->conserver_connect ($system,$1,$conserver_port,$conuser);
         } else {
-                return($port) unless $port=~/^\d+/;
+                if ($port !~ /^\d+/) {
+                        close $sock;
+                        return $port; # answer in $port is an error message
+                }
         }
 
 
@@ -80,18 +77,6 @@ sub conserver_connect
         return($sock);
 }
 
-=head2 conserver_disconnect
-
-Disconnect the filehandle given as first argument from the conserver.
-We first try to quit kindly but if this fails (by what reason ever)
-the filehandle is simply closed. Closing a socket can not fail, so the
-function always succeeds. Thus no return value is needed.
-
-@param  IO::Socket::INET - file handle connected to the conserver
-
-@return none
-
-=cut
 
 sub conserver_disconnect
 {
@@ -111,17 +96,6 @@ sub conserver_disconnect
 }
 
 
-=head2 start_simnow
-
-Start a simnow installation on given host. Installer is supposed to
-start the simnow controller in turn.
-
-@param string - hostname
-
-@return success - 0
-@return error   - error string
-
-=cut
 
 sub start_simnow
 {
@@ -139,62 +113,53 @@ sub start_simnow
 }
 
 
-=head2 reboot_system
 
-Reboot the named system. First we try to do it softly, if that does not
-work, we try a hard reboot. Unfortunately this does not give any
-feedback. Thus you have to wait for the typical reboot time of the
-system in question and if the system does not react after this time
-assume that the reboot failed. This is not included in this function,
-since it would make it to complex.
+sub start_ssh
+{
+        my ($self, $hostname) = @_;
+
+        my $tapper_script = $self->cfg->{files}{tapper_prc};
+        my $tftp_host = $self->cfg->{mcp_host};
+        my $error = Net::SSH::ssh("$hostname","$tapper_script --host $tftp_host");
+        return "Can not start PRC with ssh: $error" if $error;
+        return 0;
+}
+
+
+sub install_client_package
+{
+        my ($self, $hostname, $package) = @_;
+
+        my $dest_path  = $package->{dest_path} || '/tmp';
+        $dest_path .= "/tapper-clientpkg.tgz";
+
+        my $arch = $package->{arch};
+        return "No architecture defined. Can not install client package" if not $arch;
+        my $clientpkg = $self->cfg->{files}{tapper_package}{$arch};
+
+        $clientpkg = $self->cfg->{paths}{package_dir}.$clientpkg
+          if not $clientpkg =~ m,^/,;
+
+        my $scp     = Net::SCP->new($hostname);
+        my $success = $scp->put(
+                               $clientpkg,
+                               $dest_path,
+                              );
+        return "Can not copy client package '$clientpkg' to $hostname:/$dest_path: ".$scp->{errstr} if not $success;
+
+        my $error = Net::SSH::ssh("$hostname","tar -xzf $dest_path -C /");
+        return "Can not unpack client package on $hostname: $!" if $error;
+        return 0;
+}
 
-@param string - name of the system to be rebooted
-@param bool   - hard reset without ssh
 
-@return success - 0
-@return error   - error string
 
-=cut
 
 sub reboot_system
 {
         my ($self, $host, $hard) = @_;
-	$self->log->debug("Trying to reboot $host.");
+        $self->log->debug("Trying to reboot $host.");
 
-        ## Some machines do not boot up correctly after a shutdown with
-        ## ssh and reboot (e.g. because they do not even shut down correctly
-        ## waiting for services like NFS to shut down).
-        if (not $hard) {
-                $self->log->info("Try reboot via Net::SSH::Expect"); # usually for the installed host/dom0 system
-                my $ssh = new Net::SSH::Expect( host     => $host,
-                                                password => 'xyzxyz',
-                                                user     => 'root',
-                                                raw_pty  => 1 );
-                my $output;
-                # Try login, with timeout
-                eval {
-                        local $SIG{ALRM} = sub{ die("timeout in login") };
-                        alarm(10);
-                        my $login_output = $ssh->login();
-
-                        if ($login_output)
-                        {
-                                $self->log->info("Logged in. Try exec reboot");
-                                $ssh->exec("stty raw -echo");
-                                $output = $ssh->exec("reboot");
-                        }
-                };
-                alarm(0);
-                if ($output and $output =~ 'The system is going down for reboot') {
-                        return 0;
-                } elsif ($@) {
-                        $self->log->error("Can not reboot $host with SSH: $@");
-                } else {
-                        $self->log->error("Can not reboot $host with SSH: $output");
-                }
-        }
-
-        # else trigger reset switch
 
         my $reset_plugin         = $self->cfg->{reset_plugin};
         my $reset_plugin_options = $self->cfg->{reset_plugin_options};
@@ -206,8 +171,9 @@ sub reboot_system
                 return "Could not load $reset_class";
         } else {
                 no strict 'refs'; ## no critic
-                $self->log->info("Call ${reset_class}::reset_host($host)");
-                my ($error, $retval) = &{"${reset_class}::reset_host"}($self, $host, $reset_plugin_options);
+                $self->log->debug("Call $reset_class->reset_host($host, $reset_plugin_options)");
+                my $reset_object = $reset_class->new();
+                my ($error, $retval) = $reset_object->reset_host($host, $reset_plugin_options);
                 if ($error) {
                         $self->log->error("Error occured: ".$retval);
                         return $retval;
@@ -217,69 +183,23 @@ sub reboot_system
 }
 
 
-=head2 write_grub_file
-
-Write a grub file for the system given as parameter. An optional second
-parameter containing the text to be put into the grub file can be used. If
-this parameter is not defined or empty a default value is used.
-
-@param string - name of the system
-@param string - text to put into grub file; optional
-
-
-@return success - 0
-@return error   - error string
-
-=cut
 
 sub write_grub_file
 {
         my ($self, $system, $text) = @_;
-        my $tapper_host = Sys::Hostname::hostname();
-        my $tapper_ip   = gethostbyname($tapper_host);
-        return qq{Can not find IP address of "$tapper_host".} if not $tapper_ip;
-        $tapper_ip = inet_ntoa($tapper_ip);
+        return "No grub text given" unless $text;
 
         my $grub_file    = $self->cfg->{paths}{grubpath}."/$system.lst";
+        $self->log->debug("writing grub file $grub_file");
 
-	$self->log->debug("writing grub file ($tapper_host, $grub_file)");
-
-	# create the initial grub file for installation of the test system,
-	open (my $GRUBFILE, ">", $grub_file) or return "Can open ".$self->cfg->{paths}{grubpath}."/$system.lst for writing: $!";
-
-        my $tftp_server = $self->cfg->{tftp_server_address};
-        my $kernel = $self->cfg->{files}{installer_kernel};
-        my $nfsroot = $self->cfg->{paths}{nfsroot};
-	if (not $text) {
-                $text = <<END;
-serial --unit=0 --speed=115200
-terminal serial
-
-default 0
-timeout 2
-
-title Test
-     tftpserver $tftp_server
-     kernel $kernel earlyprintk=serial,ttyS0,115200 console=ttyS0,115200 root=/dev/nfs ro ip=dhcp nfsroot=$nfsroot tapper_host=$tapper_host tapper_ip=$tapper_ip
-END
-        }
-	print $GRUBFILE $text;
-	close $GRUBFILE or return "Can't save grub file for $system:$!";
-	return(0);
+        # create the initial grub file for installation of the test system,
+        open (my $GRUBFILE, ">", $grub_file) or return "Can open ".$self->cfg->{paths}{grubpath}."/$system.lst for writing: $!";
+        print $GRUBFILE $text;
+        close $GRUBFILE or return "Can't save grub file for $system:$!";
+        return(0);
 }
 
 
-=head2 hw_report_create
-
-Create a report containing the test machines hw config as set in the hardware
-db. Leave the sending to caller
-
-@param int - testrun id
-
-@return success - (0, hw_report)
-@return error   - (1, error string)
-
-=cut
 
 sub hw_report_create
 {
@@ -313,3 +233,115 @@ ok 2 - Sending
 }
 
 1;
+
+__END__
+=pod
+
+=encoding utf-8
+
+=head1 NAME
+
+Tapper::MCP::Net
+
+=head2 conserver_connect
+
+This function opens a connection to the conserver. Conserver, port and user
+can be given as arguments, yet are optional.
+@param string - system to open a console to
+@opt   string - Address or name of the console server
+@opt   int    - port number of the console server
+@opt   string - username to be used
+
+@returnlist success - (IO::Socket::INET object)
+@returnlist error   - (error string)
+
+=head2 conserver_disconnect
+
+Disconnect the filehandle given as first argument from the conserver.
+We first try to quit kindly but if this fails (by what reason ever)
+the filehandle is simply closed. Closing a socket can not fail, so the
+function always succeeds. Thus no return value is needed.
+
+@param  IO::Socket::INET - file handle connected to the conserver
+
+@return none
+
+=head2 start_simnow
+
+Start a simnow installation on given host. Installer is supposed to
+start the simnow controller in turn.
+
+@param string - hostname
+
+@return success - 0
+@return error   - error string
+
+=head2 start_ssh
+
+Start a ssh testrun on given host. This starts both the Installer and PRC.
+
+@param string - hostname
+
+@return success - 0
+@return error   - error string
+
+=head2 install_client_package
+
+Install client package of given architecture on given host at optional
+given possition.
+
+@param string   - hostname
+@param hash ref - contains arch and dest_path
+
+@return success - 0
+@return error   - error string
+
+=head2 reboot_system
+
+Reboot the named system. First we try to do it softly, if that does not
+work, we try a hard reboot. Unfortunately this does not give any
+feedback. Thus you have to wait for the typical reboot time of the
+system in question and if the system does not react after this time
+assume that the reboot failed. This is not included in this function,
+since it would make it to complex.
+
+@param string - name of the system to be rebooted
+@param bool   - hard reset without ssh
+
+@return success - 0
+@return error   - error string
+
+=head2 write_grub_file
+
+Write the given text to the grub file for the system given as parameter.
+
+@param string - name of the system
+@param string - text to put into grub file
+
+@return success - 0
+@return error   - error string
+
+=head2 hw_report_create
+
+Create a report containing the test machines hw config as set in the hardware
+db. Leave the sending to caller
+
+@param int - testrun id
+
+@return success - (0, hw_report)
+@return error   - (1, error string)
+
+=head1 AUTHOR
+
+AMD OSRC Tapper Team <tapper@amd64.org>
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is Copyright (c) 2012 by Advanced Micro Devices, Inc..
+
+This is free software, licensed under:
+
+  The (two-clause) FreeBSD License
+
+=cut
+
