@@ -1,10 +1,15 @@
-use MooseX::Declare;
-
-use 5.010;
-
 ## no critic (RequireUseStrict)
-class Tapper::MCP::Scheduler::Algorithm with MooseX::Traits {
+package Tapper::MCP::Scheduler::Algorithm;
+BEGIN {
+  $Tapper::MCP::Scheduler::Algorithm::AUTHORITY = 'cpan:AMD';
+}
+{
+  $Tapper::MCP::Scheduler::Algorithm::VERSION = '4.0.1';
+}
+# ABSTRACT: name of the queue has to be unique
 
+        use 5.010;
+        use Moose;
         use Tapper::Model 'model';
 
         has queues => (
@@ -13,10 +18,15 @@ class Tapper::MCP::Scheduler::Algorithm with MooseX::Traits {
                        default    => sub { model('TestrunDB')->resultset('Queue')->official_queuelist },
                       );
 
-        method queue_count { scalar keys %{$self->queues} }
+        sub queue_count {
+                my ($self) = @_;
+
+                scalar keys %{$self->queues}
+        }
+
+        sub add_queue {
+                my ($self,  $q) = @_;
 
-        method add_queue( $q) # Queue
-        {
                 my $qname = $q->name;
                 if ($self->queues->{$qname}) {
                         warn "Queue with name '$qname' already exists";
@@ -31,38 +41,51 @@ class Tapper::MCP::Scheduler::Algorithm with MooseX::Traits {
                 $self->queues->{$qname} = $q;
         }
 
-        method remove_queue( $q) { # Queue
+        sub remove_queue {
+                my ($self,  $q) = @_;
                 delete $self->queues->{$q->name};
         }
 
-        method update_queue( $q) { # Queue
+        sub update_queue {
+                my ($self,  $q) = @_;
                 # interface
                 die "Interface update_queue not implemented";
         }
 
-        method lookup_next_queue() {
+        sub lookup_next_queue {
+                my ($self) = @_;
                 # interface
                 die "Interface lookup_next_queue not implemented";
         }
 
-        method get_next_queue() {
+        sub get_next_queue {
+                my ($self) = @_;
                 # interface
                 die "Interface get_next_queue not implemented";
         }
-}
 
-__END__
+        with 'MooseX::Traits';
+1;
+
+
+
+=pod
+
+=encoding utf-8
+
+=head1 NAME
+
+Tapper::MCP::Scheduler::Algorithm - name of the queue has to be unique
 
 =head2 add_queue
 
 Add a new queue to the scheduler.
 
-@param Scheduler::Queue - name of the queue has to be unique
+@param Scheduler::Queue -
 
 @return success - 0
 @return error   - error string
 
-
 =head2 remove_queue
 
 Remove a queue from scheduling
@@ -72,7 +95,6 @@ Remove a queue from scheduling
 @return success - 0
 @return error   - error string
 
-
 =head2 update_queue
 
 Update the time entry of the given queue
@@ -81,4 +103,20 @@ Update the time entry of the given queue
 
 @return success - 0
 
+=head1 AUTHOR
+
+AMD OSRC Tapper Team <tapper@amd64.org>
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is Copyright (c) 2012 by Advanced Micro Devices, Inc..
+
+This is free software, licensed under:
+
+  The (two-clause) FreeBSD License
+
 =cut
+
+
+__END__
+
