@@ -1,14 +1,22 @@
-use MooseX::Declare;
-
-use 5.010;
-
 ## no critic (RequireUseStrict)
-class Tapper::MCP::Scheduler::PrioQueue
+package Tapper::MCP::Scheduler::PrioQueue;
+BEGIN {
+  $Tapper::MCP::Scheduler::PrioQueue::AUTHORITY = 'cpan:AMD';
+}
 {
+  $Tapper::MCP::Scheduler::PrioQueue::VERSION = '4.0.1';
+}
+# ABSTRACT: Object for test queue abstraction
+
+        use 5.010;
+        use Moose;
+
         use Tapper::Model 'model';
         use aliased 'Tapper::Schema::TestrunDB::Result::TestrunScheduling';
 
-        method _max_seq {
+        sub _max_seq {
+                my ($self) = @_;
+
                 my $job_with_max_seq = model('TestrunDB')->resultset('TestrunScheduling')->search
                     (
                      { prioqueue_seq => { '>', 0 } },
@@ -21,15 +29,17 @@ class Tapper::MCP::Scheduler::PrioQueue
                 return 0;
         }
 
-        method add($job, $is_subtestrun?)
-        {
+        sub add {
+                my ($self, $job, $is_subtestrun) = @_;
+
                 my $max_seq = $self->_max_seq;
                 $job->prioqueue_seq($max_seq + 1);
                 $job->update;
         }
 
-        method get_testrequests # get_jobs
-        {
+        sub get_testrequests {
+                my ($self) = @_;
+
                 no strict 'refs'; ## no critic (ProhibitNoStrict)
                 my $testrequests_rs = model('TestrunDB')->resultset('TestrunScheduling')->search
                     ({
@@ -42,7 +52,9 @@ class Tapper::MCP::Scheduler::PrioQueue
                 return $testrequests_rs;
         }
 
-        method get_first_fitting($free_hosts) {
+        sub get_first_fitting {
+                my ($self, $free_hosts) = @_;
+
                 my $jobs = $self->get_testrequests;
                 while (my $job = $jobs->next()) {
                         if (my $host = $job->fits($free_hosts)) {
@@ -58,16 +70,14 @@ class Tapper::MCP::Scheduler::PrioQueue
                 }
                 return;
         }
-}
-
-{
-        # help the CPAN indexer
-        package Tapper::MCP::Scheduler::PrioQueue;
-}
 
 1;
 
-__END__
+
+
+=pod
+
+=encoding utf-8
 
 =head1 NAME
 
@@ -88,7 +98,6 @@ Get a testrequest for one of the free hosts provided as parameter.
 
 =head2 produce
 
-
 Call the producer method associated with this object.
 
 @param string - hostname
@@ -96,19 +105,20 @@ Call the producer method associated with this object.
 @return success - test run id
 @return error   - exception
 
-
-
 =head1 AUTHOR
 
-Maik Hentsche, C<< <maik.hentsche at amd.com> >>
+AMD OSRC Tapper Team <tapper@amd64.org>
 
-=head1 COPYRIGHT & LICENSE
+=head1 COPYRIGHT AND LICENSE
 
-Copyright 2008-2011 AMD OSRC Tapper Team, all rights reserved.
+This software is Copyright (c) 2012 by Advanced Micro Devices, Inc..
 
-This program is released under the following license: freebsd
+This is free software, licensed under:
+
+  The (two-clause) FreeBSD License
 
 =cut
 
-# Idea: provide functions that map to feature has
+
+__END__
 
