@@ -1,58 +1,23 @@
 package Tapper::MCP::Startup;
+BEGIN {
+  $Tapper::MCP::Startup::AUTHORITY = 'cpan:AMD';
+}
+{
+  $Tapper::MCP::Startup::VERSION = '4.0.1';
+}
+# ABSTRACT: the central "Master Control Program" starter
+
+use 5.010;
 
 use strict;
 use warnings;
 
 use Tapper::MCP::Master;
-
 use Moose;
 
-
 no strict 'refs'; ## no critic (ProhibitNoStrict)
 
 
-=head1 NAME
-
-Tapper::MCP - the central "Master Control Program"
-
-=head1 SYNOPSIS
-
- use Tapper::MCP::Startup qw(:all);
-
-=head1 FUNCTIONS
-
-=begin method
-
-Declares a method.
-
-=end method
-
-=begin start
-
-Starts all registered daemons.
-
-=end start
-
-=begin stop
-
-Stops all registered daemons.
-
-=end stop
-
-=begin restart
-
-Restarts all registered daemons.
-
-=end restart
-
-=begin status
-
-Prints status of all registered daemons.
-
-=end status
-
-=cut
-
 has master  => (is          => 'rw',
                 default     => sub { new Tapper::MCP::Master ( pidfile => '/tmp/tapper_mcp_master.pid' ) }
                );
@@ -75,34 +40,19 @@ around 'new' => sub {
                      return $self;
                     };
 
-=begin set_servers
-
-Registers all handled daemons in an array.
-
-=end set_servers
-
-=cut
 
 sub set_servers
 {
-        my ($self) = @_; 
+        my ($self) = @_;
         $self->servers ([
                          $self->master,
                         ]);
 }
 
-=begin run
-
-Dispatches the commandline command (start, stop, restart, status) to
-all its daemons.
-
-=end run
-
-=cut
 
 sub run
 {
-        my ($self) = @_; 
+        my ($self) = @_;
         my ($command) = @ARGV;
         return unless $command && grep /^$command$/, qw(start status restart stop);
         local @ARGV;   # cleaner approach than changing @ARGV
@@ -111,27 +61,47 @@ sub run
 
 1;
 
+__END__
+=pod
+
+=encoding utf-8
+
+=head1 NAME
+
+Tapper::MCP::Startup - the central "Master Control Program" starter
+
+=head1 SYNOPSIS
+
+ use Tapper::MCP::Startup qw(:all);
+
+=head1 FUNCTIONS
+
+=for method Declares a method.
+
+=for start Starts all registered daemons.
+
+=for stop Stops all registered daemons.
+
+=for restart Restarts all registered daemons.
+
+=for status Prints status of all registered daemons.
+
+=for set_servers Registers all handled daemons in an array.
+
+=for run Dispatches the commandline command (start, stop, restart, status) to
+all its daemons.
+
 =head1 AUTHOR
 
-AMD OSRC Tapper Team, C<< <tapper at amd64.org> >>
+AMD OSRC Tapper Team <tapper@amd64.org>
 
-=head1 BUGS
+=head1 COPYRIGHT AND LICENSE
 
-None.
+This software is Copyright (c) 2012 by Advanced Micro Devices, Inc..
 
-=head1 SUPPORT
+This is free software, licensed under:
 
-You can find documentation for this module with the perldoc command.
+  The (two-clause) FreeBSD License
 
- perldoc Tapper
-
-
-=head1 ACKNOWLEDGEMENTS
-
-
-=head1 COPYRIGHT & LICENSE
-
-Copyright 2008-2011 AMD OSRC Tapper Team, all rights reserved.
-
-This program is released under the following license: freebsd
+=cut
 
